==> client-sdk-react-2.1.1.tgz: package/src/store/aiFeatures/hooks.test.ts <==
import React from 'react'
import * as reduxHooks from '../../app/hooks'
import { useAIFeatures } from './hooks'
import { waitFor } from '@testing-library/react'
import { AIFeature, AIFeatureType, AIResult } from '@zocks-communications/client-sdk-js'
import { AIFeaturesStatus } from './models'

const mockDispatch = jest.fn()
const mockSelector = jest.fn()

beforeEach(() => {
  // @ts-ignore
  jest.spyOn(React, 'useState').mockImplementation((initialState) => [initialState, jest.fn()])
  jest.spyOn(React, 'useEffect').mockImplementation((fn) => fn())
  jest.spyOn(React, 'useCallback').mockImplementation((fn) => fn)
  jest.spyOn(React, 'useMemo').mockImplementation((fn) => fn())
  jest.spyOn(reduxHooks, 'useZocksDispatch').mockReturnValue(mockDispatch)
  jest.spyOn(reduxHooks, 'useZocksStore').mockImplementation((selector) => mockSelector(selector))
})

afterEach(() => {
  jest.restoreAllMocks()
})

const aiFeatures: AIFeature[] = [
  {
    id: '1',
    name: 'Family',
    type: AIFeatureType.TOPIC_DETECTION,
    config: {
      topicMinimumConfidence: 0.5,
      topics: ['children', 'divorce', 'separation'],
    },
    executionTargets: [],
    resultTargets: [],
    metadata: '',
  },
]

const aiResults: AIResult[] = [
  {
    aiFeatureId: '1',
    createdAt: '2023-02-08T12:49:20.212613Z',
    userId: 'myUserId',
    result: {
      type: AIFeatureType.TOPIC_DETECTION,
      detectedTopic: 'children',
      confidence: 0.9,
      context: 'My son.',
    },
  },
]

test('AI features are initialized when the useAIFeatures hook is invoked', async () => {
  const mockAIFeaturesActions = {
    listAIFeatures: jest.fn().mockReturnValue(aiFeatures),
    listAIResults: jest.fn().mockReturnValue(aiResults),
  }
  mockSelector.mockReturnValueOnce({})
  mockSelector.mockReturnValueOnce(AIFeaturesStatus.INITIAL)

  useAIFeatures(mockAIFeaturesActions)
  await waitFor(() => {
    expect(mockDispatch).toHaveBeenCalledTimes(4)
  })

  expect(mockDispatch).toHaveBeenNthCalledWith(1, {
    payload: 'loading',
    type: 'aiFeature/setAIFeaturesStatus',
  })

  expect(mockDispatch).toHaveBeenNthCalledWith(3, {
    payload: 'loading',
    type: 'aiFeature/setAIFeaturesStatus',
  })

  expect(mockDispatch).toHaveBeenCalledWith({
    payload: aiFeatures,
    type: 'aiFeature/initAIFeatures',
  })
  expect(mockDispatch).toHaveBeenCalledWith({
    payload: aiResults,
    type: 'aiFeature/initAIResults',
  })
  expect(mockAIFeaturesActions.listAIFeatures).toHaveBeenCalledTimes(1)
  expect(mockAIFeaturesActions.listAIResults).toHaveBeenCalledTimes(1)
})

test('when listing the AI features fails, the error is stored and AI results are not fetched', async () => {
  const mockAIFeaturesActions = {
    listAIFeatures: jest.fn().mockRejectedValue({
      errors: [
        {
          message: 'Internal Server Error',
        },
      ],
    }),
    listAIResults: jest.fn(),
  }
  mockSelector.mockReturnValueOnce({})
  mockSelector.mockReturnValueOnce(AIFeaturesStatus.INITIAL)

  useAIFeatures(mockAIFeaturesActions)
  await waitFor(() => {
    expect(mockDispatch).toHaveBeenCalledTimes(2)
  })

  expect(mockDispatch).toHaveBeenCalledWith({
    payload: 'loading',
    type: 'aiFeature/setAIFeaturesStatus',
  })

  expect(mockDispatch).toHaveBeenCalledWith({
    payload: {
      errors: [
        {
          message: 'Internal Server Error',
        },
      ],
    },
    type: 'aiFeature/setAIFeaturesError',
  })

  expect(mockAIFeaturesActions.listAIFeatures).toHaveBeenCalledTimes(1)
  expect(mockAIFeaturesActions.listAIResults).toHaveBeenCalledTimes(0)
})

test('list ai features succeed but list ai results fails changes the state to error', async () => {
  const mockAIFeaturesActions = {
    listAIFeatures: jest.fn().mockReturnValue(aiFeatures),
    listAIResults: jest.fn().mockRejectedValue({
      errors: [
        {
          message: 'Internal Server Error',
        },
      ],
    }),
  }
  mockSelector.mockReturnValueOnce({})
  mockSelector.mockReturnValueOnce(AIFeaturesStatus.INITIAL)

  useAIFeatures(mockAIFeaturesActions)
  await waitFor(() => {
    expect(mockDispatch).toHaveBeenCalledTimes(4)
  })

  expect(mockDispatch).toHaveBeenNthCalledWith(1, {
    payload: 'loading',
    type: 'aiFeature/setAIFeaturesStatus',
  })

  expect(mockDispatch).toHaveBeenNthCalledWith(3, {
    payload: 'loading',
    type: 'aiFeature/setAIFeaturesStatus',
  })

  expect(mockDispatch).toHaveBeenCalledWith({
    payload: aiFeatures,
    type: 'aiFeature/initAIFeatures',
  })

  expect(mockDispatch).toHaveBeenCalledWith({
    payload: {
      errors: [
        {
          message: 'Internal Server Error',
        },
      ],
    },
    type: 'aiFeature/setAIFeaturesError',
  })

  expect(mockAIFeaturesActions.listAIFeatures).toHaveBeenCalledTimes(1)
  expect(mockAIFeaturesActions.listAIResults).toHaveBeenCalledTimes(1)
})

test('resetAIFeatures clears the state', async () => {
  const mockAIFeaturesActions = {
    listAIFeatures: jest.fn(),
    listAIResults: jest.fn(),
  }
  mockSelector.mockReturnValueOnce({})
  mockSelector.mockReturnValueOnce(AIFeaturesStatus.INITIAL)

  const { resetAIFeatures } = useAIFeatures(mockAIFeaturesActions)
  resetAIFeatures()

  expect(mockDispatch).toHaveBeenCalledWith({
    type: 'aiFeature/clearAIFeatures',
  })
})

==> client-sdk-react-2.13.0.tgz: package/src/store/aiFeatures/hooks.test.ts <==
import React from 'react'
import * as reduxHooks from '../../app/hooks'
import { useAIFeatures } from './hooks'
import { waitFor } from '@testing-library/react'
import { AIFeature, AIFeatureType, AIResult } from '@zocks-communications/client-sdk-js'
import { AIFeaturesStatus } from './models'

const mockDispatch = jest.fn()
const mockSelector = jest.fn()

beforeEach(() => {
  // @ts-ignore
  jest.spyOn(React, 'useState').mockImplementation((initialState) => [initialState, jest.fn()])
  jest.spyOn(React, 'useEffect').mockImplementation((fn) => fn())
  jest.spyOn(React, 'useCallback').mockImplementation((fn) => fn)
  jest.spyOn(React, 'useMemo').mockImplementation((fn) => fn())
  jest.spyOn(reduxHooks, 'useZocksDispatch').mockReturnValue(mockDispatch)
  jest.spyOn(reduxHooks, 'useZocksStore').mockImplementation((selector) => mockSelector(selector))
})

afterEach(() => {
  jest.restoreAllMocks()
})

const aiFeatures: AIFeature[] = [
  {
    id: '1',
    name: 'Family',
    type: AIFeatureType.TOPIC_DETECTION,
    config: {
      topicMinimumConfidence: 0.5,
      topics: ['children', 'divorce', 'separation'],
    },
    executionTargets: [],
    resultTargets: [],
    metadata: '',
  },
]

const aiResults: AIResult[] = [
  {
    aiFeatureId: '1',
    createdAt: '2023-02-08T12:49:20.212613Z',
    userId: 'myUserId',
    result: {
      type: AIFeatureType.TOPIC_DETECTION,
      detectedTopic: 'children',
      confidence: 0.9,
      context: 'My son.',
    },
  },
]

test('AI features are initialized when the useAIFeatures hook is invoked', async () => {
  const mockAIActions = {
    listAIFeatures: jest.fn().mockReturnValue(aiFeatures),
    listAIResults: jest.fn().mockReturnValue(aiResults),
  }
  mockSelector.mockReturnValueOnce({})
  mockSelector.mockReturnValueOnce(AIFeaturesStatus.INITIAL)

  useAIFeatures(mockAIActions)
  await waitFor(() => {
    expect(mockDispatch).toHaveBeenCalledTimes(4)
  })

  expect(mockDispatch).toHaveBeenNthCalledWith(1, {
    payload: 'loading',
    type: 'aiFeature/setAIFeaturesStatus',
  })

  expect(mockDispatch).toHaveBeenNthCalledWith(3, {
    payload: 'loading',
    type: 'aiFeature/setAIFeaturesStatus',
  })

  expect(mockDispatch).toHaveBeenCalledWith({
    payload: aiFeatures,
    type: 'aiFeature/initAIFeatures',
  })
  expect(mockDispatch).toHaveBeenCalledWith({
    payload: aiResults,
    type: 'aiFeature/initAIResults',
  })
  expect(mockAIActions.listAIFeatures).toHaveBeenCalledTimes(1)
  expect(mockAIActions.listAIResults).toHaveBeenCalledTimes(1)
})

test('when listing the AI features fails, the error is stored and AI results are not fetched', async () => {
  const mockAIActions = {
    listAIFeatures: jest.fn().mockRejectedValue({
      errors: [
        {
          message: 'Internal Server Error',
        },
      ],
    }),
    listAIResults: jest.fn(),
  }
  mockSelector.mockReturnValueOnce({})
  mockSelector.mockReturnValueOnce(AIFeaturesStatus.INITIAL)

  useAIFeatures(mockAIActions)
  await waitFor(() => {
    expect(mockDispatch).toHaveBeenCalledTimes(2)
  })

  expect(mockDispatch).toHaveBeenCalledWith({
    payload: 'loading',
    type: 'aiFeature/setAIFeaturesStatus',
  })

  expect(mockDispatch).toHaveBeenCalledWith({
    payload: {
      errors: [
        {
          message: 'Internal Server Error',
        },
      ],
    },
    type: 'aiFeature/setAIFeaturesError',
  })

  expect(mockAIActions.listAIFeatures).toHaveBeenCalledTimes(1)
  expect(mockAIActions.listAIResults).toHaveBeenCalledTimes(0)
})

test('when listing the AI features succeeds, but listing the AI results fails, the error is stored', async () => {
  const mockAIActions = {
    listAIFeatures: jest.fn().mockReturnValue(aiFeatures),
    listAIResults: jest.fn().mockRejectedValue({
      errors: [
        {
          message: 'Internal Server Error',
        },
      ],
    }),
  }
  mockSelector.mockReturnValueOnce({})
  mockSelector.mockReturnValueOnce(AIFeaturesStatus.INITIAL)

  useAIFeatures(mockAIActions)
  await waitFor(() => {
    expect(mockDispatch).toHaveBeenCalledTimes(4)
  })

  expect(mockDispatch).toHaveBeenNthCalledWith(1, {
    payload: 'loading',
    type: 'aiFeature/setAIFeaturesStatus',
  })

  expect(mockDispatch).toHaveBeenNthCalledWith(3, {
    payload: 'loading',
    type: 'aiFeature/setAIFeaturesStatus',
  })

  expect(mockDispatch).toHaveBeenCalledWith({
    payload: aiFeatures,
    type: 'aiFeature/initAIFeatures',
  })

  expect(mockDispatch).toHaveBeenCalledWith({
    payload: {
      errors: [
        {
          message: 'Internal Server Error',
        },
      ],
    },
    type: 'aiFeature/setAIFeaturesError',
  })

  expect(mockAIActions.listAIFeatures).toHaveBeenCalledTimes(1)
  expect(mockAIActions.listAIResults).toHaveBeenCalledTimes(1)
})

test('resetAIFeatures clears the state', async () => {
  const mockAIActions = {
    listAIFeatures: jest.fn(),
    listAIResults: jest.fn(),
  }
  mockSelector.mockReturnValueOnce({})
  mockSelector.mockReturnValueOnce(AIFeaturesStatus.INITIAL)

  const { resetAIFeatures } = useAIFeatures(mockAIActions)
  resetAIFeatures()

  expect(mockDispatch).toHaveBeenCalledWith({
    type: 'aiFeature/clearAIFeatures',
  })
})
